﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Newtonsoft.Json;
using RetailForce.Common.Validation;
using RetailForce.Fiscalisation;
using RetailForce.Fiscalisation.Configuration;
using RetailForce.Fiscalisation.Implementation.Germany;
using RetailForce.Fiscalisation.Model;
using RetailForce.Fiscalisation.Model.Document;
using RetailForce.Fiscalisation.Samples.FiscalServiceClientSample.Model;
using RetailForce.Fiscalisation.ServiceClient;

namespace RetailForce.Fiscalisation.Samples.FiscalServiceClientSample
{
    public partial class SampleForm : Form
    {
        private const string _fiscalClientUrl = "http://localhost:7678/api/v1/";
        private FiscalServiceClient _serviceClient = new FiscalServiceClient(_fiscalClientUrl);
        private FiscalClient _fiscalClient;
        private SimpleCustomer _simpleCustomer;

        public SampleForm()
        {
            InitializeComponent();

            cmbClient.Items.Add(new FiscalClient() { StoreName = "Empty" });

            try
            {
                FiscalClient[] clients = _serviceClient.GetClients();
                cmbClient.Items.AddRange(clients);
                List<DocumentType> documentTypes = Enum.GetValues(typeof(DocumentType)).Cast<DocumentType>().ToList();
                cmbDocType.Items.AddRange(documentTypes.Cast<Object>().ToArray());
                cmbDocType.SelectedItem = DocumentType.Receipt;
                List<BusinessTransactionType> businessTransactionTypes = Enum.GetValues(typeof(RetailForce.Fiscalisation.Model.BusinessTransactionType))
                    .Cast<BusinessTransactionType>()
                    .Where(w => w != BusinessTransactionType.Revenue && w != BusinessTransactionType.Discount)
                    .ToList();
                cmbBusinessTransactionType.Items.AddRange(businessTransactionTypes.Cast<Object>().ToArray());
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show($"Error loading fiscal clients from service at address '{_fiscalClientUrl}'.\n\n{ex.Message}");
            }
        }

        public void ValidateDocument()
        {
            if (_fiscalClient == null)
            {
                MessageBox.Show("Select client to communicate first.");
                return;
            }

            Document document = GetDocument();

            document.FiscalDocumentNumber = 2;
            document.FiscalDocumentRevision = 1;
            document.FiscalDocumentStartTime = 10000;
            List<ValidationError> errors = _serviceClient.ValidateDocument(document);

            lstValidationError.Items.Clear();
            if (errors != null && errors.Count > 0)
            {
                lstValidationError.Items.AddRange(errors.ToArray());
            }
            else MessageBox.Show("Validation successful.");
        }

        public FiscalResponse StoreDocument()
        {
            if (_fiscalClient == null) throw new NullReferenceException("fiscal client must be set.");

            Document document = GetDocument();

            FiscalResponse fiscalResponse = _serviceClient.CreateDocument(_fiscalClient.UniqueClientId, document.DocumentType);

            document.FiscalDocumentNumber = fiscalResponse.FiscalisationDocumentNumber;
            document.FiscalDocumentRevision = fiscalResponse.FiscalisationDocumentRevision;
            document.FiscalDocumentStartTime = fiscalResponse.FiscalDocumentStartTime;

            return _serviceClient.StoreDocument(document);
        }

        /// <summary>
        /// Creates the document from the given parameters.
        /// </summary>
        /// <returns></returns>
        public Document GetDocument()
        {
            List<string> allocGroups = null;
            if (cmbTable.SelectedItem != null && !string.IsNullOrEmpty(cmbTable.SelectedItem.ToString()))
            {
                allocGroups = new List<string>()
                {
                    cmbTable.SelectedItem.ToString()
                };
            }
            
            Document document = new Document()
            {
                CreateDate = DateTime.Now,
                DocumentType = (DocumentType)cmbDocType.SelectedItem,
                DocumentGuid = Guid.NewGuid(),
                DocumentId = DateTime.Now.ToString("O"),
                AllocationGroups = allocGroups,
                BookDate = DateTime.Now,
                IsTraining = chkTraining.Checked,
                UniqueClientId = _fiscalClient.UniqueClientId,
                User = new RetailForce.Fiscalisation.Model.User()
                {
                    Caption = txtUserCaption.Text,
                    Id = txtUserId.Text
                },
                Partner = GetPartnerFromSimpleCustomer(_simpleCustomer)
            };

            document.Positions = new List<DocumentPositionBase>();

            foreach (DocumentPositionBase item in lstPositions.Items.Cast<DocumentPositionBase>())
                document.Positions.Add(item);

            int x = 0;
            foreach (DocumentPositionBase itemPos in document.Positions)
            {
                itemPos.PositionNumber = x;
                x++;
            }

            document.Payments = new List<DocumentPayment>();
            int paymentCounter = 0;
            if (chkCash.Checked) paymentCounter++;
            if (chkCreditCard.Checked) paymentCounter++;
            if (chkEc.Checked) paymentCounter++;
            if (chkVoucher.Checked) paymentCounter++;

            decimal totalAmount = document.Positions.Where(w => w is DocumentPositionVatPosition).Cast<DocumentPositionVatPosition>().Sum(s => s.GrossValue);

            if (totalAmount != 0 && document.DocumentType != DocumentType.TableOrder)
            {
                if (paymentCounter == 0)
                {
                    document.Payments.Add(new DocumentPayment() {Amount = totalAmount, CurrencyIsoCode = "EUR", IsCash = true});
                }
                else if (paymentCounter == 1)
                {
                    if (chkCash.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount, CurrencyIsoCode = "EUR", IsCash = true});
                    if (chkCreditCard.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount, CurrencyIsoCode = "EUR", IsCash = false, Caption = "Visa"});
                    if (chkEc.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount, CurrencyIsoCode = "EUR", IsCash = false, Caption = "EC"});
                    if (chkVoucher.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount, CurrencyIsoCode = "EUR", IsCash = false, Caption = "Voucher"});
                    ;
                }
                else if (paymentCounter == 2)
                {
                    if (chkCash.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount / 2, CurrencyIsoCode = "EUR", IsCash = true});
                    if (chkCreditCard.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 2), CurrencyIsoCode = "EUR", IsCash = false, Caption = "Visa"});
                    if (chkEc.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 2), CurrencyIsoCode = "EUR", IsCash = false, Caption = "EC"});
                    if (chkVoucher.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 2), CurrencyIsoCode = "EUR", IsCash = false, Caption = "Voucher"});
                    ;
                }
                else if (paymentCounter == 3)
                {
                    if (chkCash.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount / 3, CurrencyIsoCode = "EUR", IsCash = true});
                    if (chkCreditCard.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 3), CurrencyIsoCode = "EUR", IsCash = false, Caption = "Visa"});
                    if (chkEc.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 3), CurrencyIsoCode = "EUR", IsCash = false, Caption = "EC"});
                    if (chkVoucher.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 3), CurrencyIsoCode = "EUR", IsCash = false, Caption = "Voucher"});
                    ;
                }
                else if (paymentCounter == 4)
                {
                    if (chkCash.Checked) document.Payments.Add(new DocumentPayment() {Amount = totalAmount / 4, CurrencyIsoCode = "EUR", IsCash = true});
                    if (chkCreditCard.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 4), CurrencyIsoCode = "EUR", IsCash = false, Caption = "Visa"});
                    if (chkEc.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 4), CurrencyIsoCode = "EUR", IsCash = false, Caption = "EC"});
                    if (chkVoucher.Checked) document.Payments.Add(new DocumentPayment() {Amount = GetAmount(document, paymentCounter, totalAmount, totalAmount / 4), CurrencyIsoCode = "EUR", IsCash = false, Caption = "Voucher"});
                    ;
                }
            }

            return document;
        }

        private Partner GetPartnerFromSimpleCustomer(SimpleCustomer customer)
        {
            if (customer == null) return null;

            return new Partner()
            {
                PartnerType = PartnerType.Customer,
                Caption = $"{customer.Name1} {customer.Name2}",
                Id = customer.CustomerNo,
                Street = customer.Street,
                StreetNumber = customer.StreetNumber, 
                City = customer.City,
                PostalCode = customer.PostalCode,
                CountryCode = customer.CountryCode,
                PartnerClassification = "Customer"
            };
        }

        private decimal GetAmount(Document document, int paymentCounter, decimal totalAmount, decimal partAmount)
        {
            if (document.Payments.Count == paymentCounter - 1)
                return totalAmount - document.Payments.Sum(s => s.Amount);

            return partAmount;
        }

        public DocumentPositionItem GetItemPosition(Model.SimpleItem item, int quantity, bool inHouse)
        {
            DocumentPositionItem itemPos = new DocumentPositionItem()
            {
                ItemId = item.ItemId,
                ItemCaption = item.Caption,
                GrossValue = item.Price * quantity,
                GTIN = item.EANCode,
                NetValue = Math.Round(RetailForce.Fiscalisation.Model.Vat.GetNetValue(item.Vat.VatPercent, item.Price * quantity), 2),
                TaxValue = item.Price * quantity - Math.Round(RetailForce.Fiscalisation.Model.Vat.GetNetValue(item.Vat.VatPercent, item.Price * quantity), 2),
                BaseGrossValue = item.Price * quantity,
                BaseNetValue = Math.Round(RetailForce.Fiscalisation.Model.Vat.GetNetValue(item.Vat.VatPercent, item.Price * quantity), 2),
                BaseTaxValue = item.Price * quantity - Math.Round(RetailForce.Fiscalisation.Model.Vat.GetNetValue(item.Vat.VatPercent, item.Price * quantity), 2),
                VatPercent = item.Vat.VatPercent,
                VatIdentification = item.Vat.VatId,
                Quantity = quantity,
                InHouse = inHouse,
                BusinessTransactionType = GetBusinessTransactionType(item)
            };

            return itemPos;
        }

        private BusinessTransactionType GetBusinessTransactionType(SimpleItem item)
        {
            if (item.DepositOnEmpty) return BusinessTransactionType.DepositOnEmpties;
            if (item.SinglePurposeVoucher) return BusinessTransactionType.SinglePurposeVoucher;
            if (item.MultipurposeVoucher) return BusinessTransactionType.MultiPurposeVoucher;
            if (item.CompanyTip) return BusinessTransactionType.CompanyTip;
            if (item.Tip) return BusinessTransactionType.Tip;

            return BusinessTransactionType.Revenue;
        }


        private void SampleForm_Load(object sender, EventArgs e)
        {
            
        }

        private void cmbClient_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbClient.SelectedItem != null)
            {
                _fiscalClient = (FiscalClient)cmbClient.SelectedItem;
                bool enabled = false;
                if (_fiscalClient.StoreName != "Empty")
                    enabled = true;

                cmbItem.Enabled = enabled;
                lstPositions.Enabled = enabled;
                btnAddPosition.Enabled = enabled;
                btnAddBooking.Enabled = enabled;
                btnDeletePosition.Enabled = enabled;
                btnStore.Enabled = enabled;
                btnValidate.Enabled = enabled;
                lstValidationError.Enabled = enabled;
                cmbDocType.Enabled = enabled;
                cmbBookingVat.Enabled = enabled;
                btnAddBooking.Enabled = enabled;
                txtBookingCaption.Enabled = enabled;
                cmbBusinessTransactionType.Enabled = enabled;
                btnClientStatus.Enabled = enabled;
                btnShowJson.Enabled = enabled;
                txtUserCaption.Enabled = enabled;
                txtUserId.Enabled = enabled;
                txtQuantity.Enabled = enabled;
                chkTraining.Enabled = enabled;
                chkCash.Enabled = enabled;
                chkEc.Enabled = enabled;
                chkVoucher.Enabled = enabled;
                chkCreditCard.Enabled = enabled;
                btnDiscount.Enabled = enabled;
                txtBookingValue.Enabled = enabled;
                btnAddPartner.Enabled = enabled;
                btnSupplemental.Enabled = enabled;
                btnRemovePartner.Enabled = enabled;
                btnSubTotal.Enabled = enabled;
                btnClearDocument.Enabled = enabled;
                chkInhouse.Enabled = enabled;
                btnEndofday.Enabled = enabled;
                cmbTable.Enabled = enabled;

                cmbItem.Items.Clear();
                if (enabled)
                    cmbItem.Items.AddRange(_GetItems());

                cmbBookingVat.Items.Clear();
                if (enabled)
                    cmbBookingVat.Items.AddRange(_serviceClient.GetSupportedVatDefinitions(_fiscalClient.UniqueClientId));
            }
        }

        private Model.SimpleItem[] _GetItems()
        {
            if (_fiscalClient == null || _fiscalClient.StoreName == "Empty") return null;

            Model.SimpleVat countryVat;
            switch(_fiscalClient.FiscalCountry)
            {
                case FiscalCountry.Germany: countryVat = new Model.SimpleVat() { }; break;
                case FiscalCountry.Austria: countryVat = new Model.SimpleVat() { }; break;
                default: return null;
            }

            List<Model.SimpleItem> simpleItems = new List<Model.SimpleItem>();
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000452", Caption = "Flower blue", Price = 4.50m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000714", Caption = "Flower green", Price = 4.30m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000322", Caption = "Rose", Price = 6.70m, Vat = _GetVat(noVat: true) });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "000169", Caption = "Tulip", Price = 2.30m, Vat = _GetVat() });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "001180", Caption = "Wedding bouquet", Price = 22.80m, Vat = _GetVat(), EANCode = "8718274490558" });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "003145", Caption = "Chocolate", Price = 3.4m, Vat = _GetVat(true), EANCode = "7622300429331" });
            
            simpleItems.Add(new Model.SimpleItem() { ItemId = "900001", Caption = "Voucher 10 EUR", Price = 10m, Vat = _GetVat(noVat: true), MultipurposeVoucher = true });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "910001", Caption = "Voucher for chocolate 003145", Price = 3.4m, Vat = _GetVat(true), SinglePurposeVoucher = true });

            simpleItems.Add(new Model.SimpleItem() { ItemId = "950001", Caption = "Tip", Price = 5m, Vat = _GetVat(), Tip = true });
            simpleItems.Add(new Model.SimpleItem() { ItemId = "960001", Caption = "Tip for company", Price = 5m, Vat = _GetVat(), CompanyTip = true });

            simpleItems.Add(new Model.SimpleItem() { ItemId = "990001", Caption = "6er box empties", Price = 12.2m, Vat = _GetVat(), DepositOnEmpty = true });

            return simpleItems.ToArray();
        }

        private Model.SimpleVat _GetVat(bool lowVat = false, bool noVat = false)
        {
            if (_fiscalClient == null || _fiscalClient.StoreName == "Empty") return null;

            if (!lowVat && !noVat)
            {
                switch (_fiscalClient.FiscalCountry)
                {
                    case FiscalCountry.Germany: return new Model.SimpleVat() {VatId = 1, VatPercent = 19};
                    case FiscalCountry.Austria: return new Model.SimpleVat() { VatId = 1, VatPercent = 20 };
                    default: return null;
                }
            }

            if (noVat)
            {
                switch (_fiscalClient.FiscalCountry)
                {
                    case FiscalCountry.Germany: return new Model.SimpleVat() { VatId = 5, VatPercent = 0 };
                    case FiscalCountry.Austria: return new Model.SimpleVat() { VatId = 4, VatPercent = 0 };
                    default: return null;
                }
            }
            switch (_fiscalClient.FiscalCountry)
            {
                case FiscalCountry.Germany: return new Model.SimpleVat() { VatId = 2, VatPercent = 7 };
                case FiscalCountry.Austria: return new Model.SimpleVat() { VatId = 2, VatPercent = 10 };
                default: return null;
            }
        }

        private void btnAddPosition_Click(object sender, EventArgs e)
        {
            if (cmbItem.SelectedItem != null)
            {
                lstPositions.Items.Add(GetItemPosition((SimpleItem) cmbItem.SelectedItem, Convert.ToInt32(txtQuantity.Value), chkInhouse.Checked));
                txtQuantity.Value = 1;
                chkInhouse.Checked = false;
                lblTotal.Text = GetTotalSum();
            }
        }

        private void btnDeletePosition_Click(object sender, EventArgs e)
        {
            if (lstPositions.SelectedIndex >= 0)
            {
                lstPositions.Items.RemoveAt(lstPositions.SelectedIndex);
                lblTotal.Text = GetTotalSum();
            }
        }

        private void btnValidate_Click(object sender, EventArgs e)
        {
            ValidateDocument();
        }

        private void btnStore_Click(object sender, EventArgs e)
        {
            if (_fiscalClient == null)
            {
                MessageBox.Show("Select client to communicate first.");
                return;
            }

            FiscalResponse fiscalResponse = StoreDocument();
            btnClearDocument_Click(sender, e);

            if (fiscalResponse == null) return;

            if (fiscalResponse.FiscalCountry == FiscalCountry.Germany)
            {
                lstValidationError.Items.Clear();
                lstValidationError.Items.Add($"ProcessType: {fiscalResponse.ProcessType()}");
                lstValidationError.Items.Add($"ProcessData: {fiscalResponse.ProcessData()}");
                lstValidationError.Items.Add($"StartTime: {fiscalResponse.TransactionStartTime()}");
                lstValidationError.Items.Add($"EndTime: {fiscalResponse.TransactionEndTime()}");
                lstValidationError.Items.Add($"DocumentNumber: {fiscalResponse.FiscalisationDocumentNumber}");
                lstValidationError.Items.Add($"DocumentRevision: {fiscalResponse.FiscalisationDocumentRevision}");
                lstValidationError.Items.Add($"QrCodeData: {fiscalResponse.QrCodeDataString()}");
                lstValidationError.Items.Add($"Signature: {fiscalResponse.Signature}");

                // try to get qr code
                string qrCode = fiscalResponse.QrCodeDataString();

                string qrCodePicture = _serviceClient.GetQrCodeBase64(qrCode);

                if (!string.IsNullOrEmpty(qrCodePicture))
                {
                    Image image;
                    byte[] bytes = Convert.FromBase64String(qrCodePicture);
                    using (MemoryStream stream = new MemoryStream(bytes))
                    {
                        image = Image.FromStream(stream);
                    }
                    picQrCode.Image = image;
                }
            }
        }

        private void btnCreateClient_Click(object sender, EventArgs e)
        {
            _serviceClient.CreateClientByCloud(
                new CompanyIdentification() {Identification = "", Type = CompanyIdentification.IdentificationType.VatNumber},
                "",
                "",
                "",
                "");

            FiscalClient[] clients = _serviceClient.GetClients();

            cmbClient.Items.Clear();
            cmbClient.Items.Add(new FiscalClient() { StoreName = "Empty" });
            cmbClient.Items.AddRange(clients);
        }

        private void cmbItem_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void btnShowJson_Click(object sender, EventArgs e)
        {
            if (_fiscalClient == null)
            {
                MessageBox.Show("Select client to communicate first.");
                return;
            }

            Document document = GetDocument();

            string documentJson = JsonConvert.SerializeObject(document, Formatting.Indented);

            txtJson.Text = documentJson;
            txtJson.Visible = !txtJson.Visible;
            btnCopyJson.Visible = txtJson.Visible;
            btnQrCode.Visible = !btnCopyJson.Visible;
            picQrCode.Visible = !btnCopyJson.Visible;
            btnEndofday.Visible = !btnCopyJson.Visible;
        }

        private void btnClientStatus_Click(object sender, EventArgs e)
        {
            FiscalClientStatus status = _serviceClient.GetClientStatus(_fiscalClient.UniqueClientId);
            if (status != null)
                MessageBox.Show($"Status: {status.StoreNumber}, {status.TerminalNumber}, {status.State}");
        }

        private void btnAddBooking_Click(object sender, EventArgs e)
        {
            DocumentPositionBooking positionBooking = new DocumentPositionBooking()
            {
                BusinessTransactionType = (BusinessTransactionType)cmbBusinessTransactionType.SelectedItem,
                Caption = txtBookingCaption.Text,
                GrossValue = txtBookingValue.Value,
                NetValue = Math.Round(Vat.GetNetValue(((Vat)cmbBookingVat.SelectedItem).VatPercents[0], txtBookingValue.Value), 2),
                TaxValue = txtBookingValue.Value - Math.Round(Vat.GetNetValue(((Vat)cmbBookingVat.SelectedItem).VatPercents[0], txtBookingValue.Value), 2),
                VatIdentification = ((Vat)cmbBookingVat.SelectedItem).VatIdentification,
                VatPercent = ((Vat)cmbBookingVat.SelectedItem).VatPercents[0]
            };

            lstPositions.Items.Add(positionBooking);

            lblTotal.Text = GetTotalSum();
        }

        private void btnDiscount_Click(object sender, EventArgs e)
        {
            if (lstPositions.SelectedIndex >= 0)
            {
                if (lstPositions.SelectedItem is DocumentPositionItem)
                {
                    DocumentPositionItem itemPos = (DocumentPositionItem) lstPositions.SelectedItem;
                    if (itemPos.Discounts.Count > 0)
                    {
                        MessageBox.Show("Cannot add second discount at position at this simple.");
                        return;
                    }


                    itemPos.Discounts.Add(new Discount()
                    {
                        Caption = "Rabatt",
                        DiscountOrder = 0,
                        Type = DiscountType.Discount,
                        TypeValue = 10,
                        DiscountValue = itemPos.BaseGrossValue * 0.1m
                    });

                    itemPos.GrossValue = itemPos.BaseGrossValue - itemPos.BaseGrossValue * 0.1m;
                    itemPos.NetValue = Math.Round(Vat.GetNetValue(itemPos.VatPercent, itemPos.GrossValue), 2);
                    itemPos.TaxValue = itemPos.GrossValue - itemPos.NetValue;

                    lstPositions.Items[lstPositions.SelectedIndex] = lstPositions.Items[lstPositions.SelectedIndex];

                    lblTotal.Text = GetTotalSum();
                }
            }
        }

        private void btnCopyJson_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(txtJson.Text);
            MessageBox.Show("Json copied successfully to the clipboard.");
        }

        private void btnQrCode_Click(object sender, EventArgs e)
        {
            picQrCode.Visible = !picQrCode.Visible;
        }

        private void btnAddPartner_Click(object sender, EventArgs e)
        {
            _simpleCustomer = new SimpleCustomer()
            {
                CustomerNo = "2241247",
                Name1 = "Franz",
                Name2 = "Österreicher",
                Street = "Fiskalisierungsstrasse",
                StreetNumber = "1",
                City = "Wien",
                PostalCode = "1090",
                CountryCode = "DEU"
            };

            lblPartner.Visible = true;
            lblPartner.Text = JsonConvert.SerializeObject(_simpleCustomer, Formatting.Indented).Replace("\"", "")
                .Replace("{", "").Replace("}","")
                .Replace(",","").Substring(2);
            lstPositions.Left = 21 + 200;
            lstPositions.Width = 629 - 200;
        }

        private void btnRemovePartner_Click(object sender, EventArgs e)
        {
            _simpleCustomer = null;
            lblPartner.Visible = false;
            lstPositions.Left = 21;
            lstPositions.Width = 629;
        }

        private void btnSupplemental_Click(object sender, EventArgs e)
        {
            if (lstPositions.SelectedIndex >= 0)
            {
                if (lstPositions.SelectedItem is DocumentPositionItem)
                {
                    DocumentPositionItem itemPos = (DocumentPositionItem)lstPositions.SelectedItem;
                    if (itemPos.Discounts.Count > 0)
                    {
                        MessageBox.Show("Cannot add second discount at position at this simple.");
                        return;
                    }


                    itemPos.Discounts.Add(new Discount()
                    {
                        Caption = "Aufschlag",
                        DiscountOrder = 0,
                        Type = DiscountType.Allowance,
                        TypeValue = 10,
                        DiscountValue = 10m
                    });

                    itemPos.GrossValue = itemPos.BaseGrossValue + 10m;
                    itemPos.NetValue = Math.Round(Vat.GetNetValue(itemPos.VatPercent, itemPos.GrossValue), 2);
                    itemPos.TaxValue = itemPos.GrossValue - itemPos.NetValue;

                    lstPositions.Items[lstPositions.SelectedIndex] = lstPositions.Items[lstPositions.SelectedIndex];
                }
            }
        }

        private void btnSubTotal_Click(object sender, EventArgs e)
        {
            DocumentPositionSubTotal positionSubTotal = new DocumentPositionSubTotal()
            {
                Value = lstPositions.Items.Cast<DocumentPositionBase>().Where(w => !(w is DocumentPositionSubTotal)).Cast<DocumentPositionVatPosition>().Sum(s => s.GrossValue),
                BaseValue = lstPositions.Items.Cast<DocumentPositionBase>().Where(w => !(w is DocumentPositionSubTotal)).Cast<DocumentPositionVatPosition>().Sum(s => s.GrossValue)
            };

            lstPositions.Items.Add(positionSubTotal);
        }

        private void btnClearDocument_Click(object sender, EventArgs e)
        {
            btnRemovePartner_Click(sender, e);
            lstPositions.Items.Clear();
            cmbTable.SelectedIndex = 0;
            lblTotal.Text = GetTotalSum();
        }

        private string GetTotalSum()
        {
            decimal total = lstPositions.Items.Cast<DocumentPositionBase>()
                .Where(w => w.Type.In(DocumentPositionType.Item, DocumentPositionType.Booking))
                .Cast<IBusinessTransactionTypePosition>().Sum(s => s.GrossValue);

            return $"{total:0.00}";
        }

        private void btnEndofday_Click(object sender, EventArgs e)
        {
            if (_fiscalClient == null)
            {
                MessageBox.Show("Select client to communicate first.");
                return;
            }

            Document document = _serviceClient.GetEndOfDayDocument(_fiscalClient.UniqueClientId);

            string documentJson = JsonConvert.SerializeObject(document, Formatting.Indented);

            txtJson.Text = documentJson;
            txtJson.Visible = !txtJson.Visible;
            btnCopyJson.Visible = txtJson.Visible;
            btnQrCode.Visible = !btnCopyJson.Visible;
            picQrCode.Visible = !btnCopyJson.Visible;
            btnEndofday.Visible = !btnCopyJson.Visible;
        }
    }
}
